#!/bin/bash

###############
# global vars #
###############

# default: fashwave.sh
myname="${0##*/}"

use_gamemode=0

# determined by nproc
threads=$(nproc)

# default: 1280x720
o_res="1280x720"

#############
# functions #
#############

_help() {
cat << __HELP__
  Usage: ${myname} [ -h | -n -d ] -i image [ -r render ] -a audio -s sroll [ -c color ] -o output
    -i image    image to serve as background
    -s sroll    sonnenrad or swastika to animate over background
                ${myname} provides both a swastika and sonnenrad render so just specify which
    -c color    optional color in 6 hex string format (#ffffff) to recolor the sonnenrad or swastika
    -r render   optional render drawn above the animated sonnenrad or swastika
    -a audio    audio track for the video, note the output video will be as long as the provided audio
    -o output   output name for the mp4 file
    -d          prints debug output for the script
    -n          dry run mode
    -h          shows script help
__HELP__
}

####################
# wrapped commands #
####################

resize_image() {
    image="$1"
     convert "$image" -resize "$o_res" res.png
}

gen_s_rot_frames() {
    s="$1"
    odir="s_frames"
    s_out="${s##*/}" # get basename
    s_out="${s_out%%.png}" # temporarely remove extension
    s_out="${odir}/${s_out}" # add output dir
    frames=18
    c=0
    while [ "$c" -lt "$frames" ]; do
        angle=$(( c * 5 ))
        num=$(printf '%02d\n' $c) # we want 0 padding
        # assuming the first frame and swastika chosen it should look like:
        #     s_frames/swastika-00.png
        s_out="${s_out}-${num}.png"
        convert \
            "$s" \
            -alpha set \( +clone -background none -rotate $angle \) \
            -gravity center -compose Src -composite \
            "$s_out"
    done
}

gen_s_color_frames() {
    color="$1"
     for img in s_frames/swastika[0-9][0-9].png; do
         convert \
             "${img}" \
             -fuzz 40% -fill "${color}" -opaque black \
             "s_frames_clr/${img##*/}"
     done
}

gen_s_alpha_frames() {
    for img in s_frames_clr/swastika[0-9][0-9].png; do
        convert \
            "${img}" \
            -alpha set -background none \
            -channel A -evaluate multiply 0.3 +channel \
            "s_trsp_frames_clr/${img##*/}"
    done
}

gen_vid_frames() {
    #swastika_trsp_frames_wht/swastika[0-9][0-9].png
    s_pattern="$1"
    #img_frames/res.png
    bg_img="$2"
    #to_gif/i_
    output="$3"
    # it may not exist
    render="$4"

    if [ -z "$render" ]; then
        for img in $s_pattern; do
            convert \
                "$bg_img" \
                \( "$img" -resize 400x400 \) -gravity North -composite \
                -resize "$o_res" "${output}/f_${img##*/}"
            done
        else
            convert \
                "$bg_img" \
                \( "$img" -resize 400x400 \) -gravity North -composite \
                "$render" -composite \
                -resize "$o_res" "${output}/f_${img##*/}"
    fi
}

frames_to_video() {
    #to_gif/i_swastika%02d.png
    frames="$1"
    audio="$2"
    output="$3"
    if [ "$use_gamemode" -eq 1 ]; then
        gamemoderun \
        ffmpeg -hide_banner -y \
            -threads "$threads" -r 24 -stream_loop -1 \
            -i "$frames" -i "$audio" \
            -c:a copy -shortest -c:v libx264 \
            -preset slower -pix_fmt yuv420p -crf 28 "$output"
    else
        ffmpeg -hide_banner -y \
            -threads "$threads" -r 24 -stream_loop -1 \
            -i "$frames" -i "$audio" \
            -c:a copy -shortest -c:v libx264 \
            -preset slower -pix_fmt yuv420p -crf 28 "$output"
    fi
}

########
# main #
########

# echo "$myname"
_help
# printf '%s\n' "$options"

